/* Filename - KONTRON_8201.c
 * Copyleft Heinz F. Lenk, heinzf_lenk@web.de, 15.12.2019
 *
 *  Testprogram for KONTRON 8201 Function Generator
 *
 *  This sample application is comprised of three basic parts:
 *
 *  1. Initialization
 *  2. Main Body
 *  3. Cleanup
 *
 *  This programm uses low level commands to control the Model 8201.
 *  The ATN-Line (attention) is controlled within the ibcmd-commands.
 *
 *  I don't know if such a construct can be made by VISA (virtual instruments).
 *  Some ibcmd commands caused ECIC error. Texting those out made no impact!
 *
 *  The last step, Cleanup, takes the board offline.

From the standard DOS shell command line, you can compile and link
the application with the Microsoft Visual C++ language interface, Gpib-32.obj,
using Microsoft Visual C++ (version 2.0 or higher), by typing in:

         cl Bdquery.c ..\Gpib-32.obj

To run the application from the DOS shell, just type in the executable
name at the prompt. To run it from within Windows choose the RUN...
option from the START menu. Enter the name of the compiled application
in the dialog box that pops up.
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

/*
 *  Include the WINDOWS.H and DECL-32.H files. The standard Windows
 *  header file, WINDOWS.H, contains definitions used by DECL-32.H and
 *  DECL-32.H contains prototypes for the GPIB routines and constants.
 *  Compile with IDE and BorlandC_gpib-32.obj  (Project -> Add to Project)
 *  or compile with bcc32 -w32 %1.c BorlandC_gpib-32.obj
 */
#include <windows.h>
#include "..\\decl-32.h"

#define ARRAYSIZE          1024     // Size of read buffer
#define ATNLINE            0x40	    // Attention Line


#define ARRAYSIZE          1024     // Size of read buffer

#define BDINDEX               0     // Board Index
#define PRIMARY_ADDR_OF_DMM   6     // Primary address of device KONTRON 8201
#define NO_SECONDARY_ADDR     0     // Secondary address of device
#define TIMEOUT               T10s  // Timeout value = 10 seconds
#define EOTMODE               1     // Enable the END message
#define EOSMODE               0     // Disable the EOS mode

int  Bd;                            // Board handle
char ReadBuffer[ARRAYSIZE + 1];     // Read Buffer
char ErrorMnemonic[21][5] = {"EDVR", "ECIC", "ENOL", "EADR", "EARG",
                             "ESAC", "EABO", "ENEB", "EDMA", "",
                             "EOIP", "ECAP", "EFSO", "", "EBUS",
                             "ESTB", "ESRQ", "", "", "", "ETAB"};
short lines;
int i;
int Dev;
short addresslist[3] = {10,NOADDR};


char STR[256];
char ch;
char Argument[10];


void GPIBCleanup(int Bd, char* ErrorMsg);
void Data_Control_Command(char* code);
void StatusTest(char Statusmask);
void SetWaveform(char* Waveform);



int main(int argc, char* argv[])
{
  if (argc == 2) strcpy(Argument,argv[1]);   // make argv[] global

  printf("copyleft Heinz F.Lenk, 13.12.2019, heinzf:lenk@web.de\n");
  printf("\nTestprogram for KONTRON 8201 Function Generator\n");
  printf("Default device address: %d\n",  PRIMARY_ADDR_OF_DMM);
  printf("To change the Model 8201's GPIB address press the LCL button,");
  printf(" then enter a number. Terminate the input with the ENT-button\n");
  printf("To show Status response use the *Argument == 'S' ");
  printf("\nPress a Key");
  scanf("%c", &ch);

/*
 * ========================================================================
 *
 * INITIALIZATION SECTION
 *
 * ========================================================================
 */

 


/*
 *  The application initializes the board using ibfind. A handle, Bd,
 *  is returned and is used in all subsequent calls to the interface
 *  board.

    int ibfind (char *udname)

ibfind is used to acquire a descriptor for an interface or user-configured device; this
interface or device descriptor can be used in subsequent traditional NI-488.2 calls.
ibfind performs the equivalent of an ibonl 1 to initialize the interface or device descriptor.
The unit descriptor returned by ibfind remains valid until the interface or device is put
offline using ibonl 0.
If ibfind is unable to get a valid descriptor, a 1 is returned; the ERR bit is set in ibsta and
iberr contains EDVR.
 */

    Bd = ibfind("GPIB0");
    if (ibsta & ERR)
    {
       GPIBCleanup(Bd, "Unable to open board");
       return 1;
    }


 /*
 * The application brings the Arbitray Function Generator online using ibdev. A
 * device handle, Dev, is returned and is used in all subsequent
 * calls to the device.

   int ibdev (int BdIndx, int pad, int sad, int tmo, int eot, int eos)

ibdev acquires a device descriptor to use in subsequent device-level traditional NI-488.2
calls. It opens and initializes a device descriptor and configures it according to the input
parameters.
For more details on the meaning and effect of each input parameter, see the corresponding
NI-488.2 calls for ibbna, ibpad, ibsad, ibtmo, ibeot, and ibeos.
 */
   Dev = ibdev(BDINDEX, PRIMARY_ADDR_OF_DMM, NO_SECONDARY_ADDR,
               TIMEOUT, EOTMODE, EOSMODE);
   if (ibsta & ERR)
   {
     printf("Unable to open device\nibsta = 0x%x iberr = %d\n",
             ibsta, iberr);
     return 1;
   }


/*
  place device online
  Set device to default values by sending SDC (Selective Device Clear)

int ibclr (int ud)

ibclr sends the GPIB Selected Device Clear (SDC) message to the device described by ud.
Set device to default values by sending SDC (Selective Device Clear)
 */
    ibclr (Dev);   // Set Remote Enable Line



/*
  8201 Address command sequence:

  Step    Command ATN state   Data Bus
  1       UNL      low          ?    0x3F
  2       MLA6     low          &    0x26
  3       SDC      low          EOT  0x04
  4                high


  8201 Device-dependent Command sequence:

  Step    Command  ATN State  Data Bus
  1       UNL      low          ?   0x3F
  2       MLA6     low          &   0x26
  3       Data     high         D   0x44  D0 (Display frequency)
  4       Data     high         0   0x30
 */

 /*
Selected Device Clear. The GPIB command used to reset internal or device
functions of an addressed Listener. See DCL.
Without this command, the 8201 will not work!
 */
    STR[0] = 0x3F;                         // UNL
    STR[1] = 0x20 + PRIMARY_ADDR_OF_DMM;   // MLAx
    STR[2] = 4;                            // SDC (Select device clear)
    STR[3] = 0;                            // String's terminator
    ibcmd (Bd,STR,strlen(STR));


    
/*   Set 8201 String Terminator

  Z0  CR,LF  w     EOI     Z5  CR     w/o   EOI
  Z1  CR,LF  w/o   EOI     Z6  LF     w     EOI
  Z2  LF,CR  w     EOI     Z7  LF     w/o   EOI
  Z3  LF,CR  w/o   EOI     Z8  no term  w   EOI
  Z4  CR     w     EOI     Z9  no term  w/o EOI
 */

   ibwrt (Dev, "Z6\r", 3L);     // set string terminator Line Feed w/o EOI



/*
  Set Function. SetWaveform is used to write all commands to the 8201
 */
  printf("\nSinus\n");
  SetWaveform("U1");
  printf("\nPress a Key");
  scanf("%c", &ch);

  printf("\nTriangular\n");
  SetWaveform("U2");
  printf("\nPress a Key");
  scanf("%c", &ch);

  printf("\nRectangular\n");
  SetWaveform("U3");
  printf("\nPress a Key");
  scanf("%c", &ch);


  printf("\nSet Frequency to 33.3kHz\n");
  SetWaveform("FR33.3E+3");
  printf("\nPress a Key");
  scanf("%c", &ch);

  printf("\nSet Frequency to 11.1kHz\n");
  SetWaveform("FR11.13E+3");
  printf("\nPress a Key");
  scanf("%c", &ch);


/*
 Read Readback Values
 N0 Frequency   N4 Pulse Witdth      N8  Leading edge        N12 Pulse error
 N1 Amplitude   N5 Pulse Delay       N9  Trailing edge       N13 error Status
 N2 Offset      N6 Trigger Periode   N10 Periode             N14 Machine Status
 N3 Symmetry    N7 Burst             N11 External Frequency
 */
    printf("\nFrequency:  ");
    Data_Control_Command("N0");

    printf("\nAmplitude:  ");
    Data_Control_Command("N1");

    printf("\nOffset:  ");
    Data_Control_Command("N2");

    printf("\nSymmetry:  ");
    Data_Control_Command("N3");

    printf("\nPulse Width:  ");
    Data_Control_Command("N4");

    printf("\nPulse Delay:  ");
    Data_Control_Command("N5");

    printf("\nTrigger Periode:  ");
    Data_Control_Command("N6");

    printf("\nBurst:  ");
    Data_Control_Command("N7");

    printf("\nLeading Edge:  ");
    Data_Control_Command("N8");

    printf("\nTrailing Edge:  ");
    Data_Control_Command("N9");

    printf("\nPeriode:  ");
    Data_Control_Command("N10");

    printf("\nExternal Frequency:  ");
    Data_Control_Command("N11");
 
    printf("\nPulse Error Data:  ");
    Data_Control_Command("N12");

    printf("\nError Status:  ");
    Data_Control_Command("N13");

    printf("\nMachine Status:  ");
    Data_Control_Command("N14");


  printf("\nPress a Key ");
  scanf("%c", &ch);





/*
 * ========================================================================
 *
 * CLEANUP SECTION
 *
 * ========================================================================
 */

/* The board is taken offline. Go to local.

    int ibloc (int ud)

Unless the REN (Remote Enable) line has been unasserted with the ibsre function, all
device-level functions automatically place the specified device in remote program mode.
ibloc is used to move devices temporarily from a remote program mode to a local mode
until the next device function is executed on that device.
 */

    ibloc (Dev);
    if (ibsta & ERR)
    {
       GPIBCleanup(Bd, "Unable to read data from device");
    }


    return 0;
}

/*
 *  After each GPIB call, the application checks whether the call
 *  succeeded. If an NI-488.2 call fails, the GPIB driver sets the
 *  corresponding bit in the global status variable. If the call
 *  failed, this procedure prints an error message, takes the board
 *  offline and exits.
 */
void GPIBCleanup(int ud, char* ErrorMsg)
{
    printf("Error : %s\nibsta = 0x%x iberr = %d (%s)\n",
            ErrorMsg, ibsta, iberr, ErrorMnemonic[iberr]);
    if (ud != -1)
    {
       printf("Cleanup: Taking board offline\n");
       ibonl (ud, 0);
    }
    printf("Press Anny's Key cleanup");
  scanf("%c", &ch);
}

void Data_Control_Command(char* code)

/* Reading from Model 8201   page: 4-33
  1   ATN active
  2   MTA6  "F"  My talk adress 6
  3   ATN false
  4   Device sends String
  5   controller reads string
  6   ATN active
  7   UNT  "_"

  If the ERR bit is set in ibsta, call GPIBCleanup with an error message.

  ibcmd (Bd, "?&" , 2L);       //  UNL MLA6
  ibwrt (Dev, "U1\r", 3L);	// set 2040. U0 = disable, U1 = sinus, U3 = rectangular
  ibcmd (Bd, "_? F" , 4L);     //  UNT UNL MLA0 MTA6   (My talk address 6)
  ibrd (Bd, ReadBuffer, ARRAYSIZE);
 */
{
   STR[0]=0x3F;                      // UNL
   STR[1]=0x20+PRIMARY_ADDR_OF_DMM;   // MLAx
   STR[2]=0;                         // String termination
   ibcmd (Bd, STR, 2L);

   strcpy(STR,code);   // make a copy
   STR[strlen(code)] = 0x0D;            // concatenate String terminator  \r
   STR[strlen(code)+1] = 0;             // Terminate String
   ibwrt (Dev, STR, strlen(STR));	// set 2040. U0 = disable, U1 = sinus, U3 = rectangular

   //    Test_Status();
    StatusTest(0x2);

   STR[0]=0x5F;                      // UNT   untalk
   STR[1]=0x3F;                      // UNL   unlisten
   STR[2]=0x20;                      // MLA0  My listener address 0
   STR[3]=0x40+PRIMARY_ADDR_OF_DMM;  // MTAx  My talk address
   STR[4]=0;                         // String termination
   ibcmd (Bd,STR,strlen(STR));    // set device to send

    ibrd (Bd, ReadBuffer, ARRAYSIZE);
    if (ibsta & ERR)
    {
       GPIBCleanup(Bd, "Unable to read data from device");
    }

    ibcmd (Bd, "_?" , 2L);       //  UNT UNL (untalk)
/*
 *  Assume that the returned string contains ASCII data. NULL terminate
 *  the string using the value in ibcntl which is the number of bytes
 *  read in. Use printf to display the string.
 */
    printf("\n%d Data read2: %s", strlen(ReadBuffer),ReadBuffer);
}



/*  Serial Polling of the 8201. page 4-20, 4.10.17
    1  Set ATN line true.
    2  Place SPE (Serial Poll Enable) "0x18" on the bus.
    3  Address the 8201 to talk.
    4  Set ATN line to false.
    5  Read the Status Byte.
    6  Set ATN low and set SPD (Serial Poll Disable) "0x19" on the bus.

    Status Byte:
    B0     Reading Done: EXT FREQ mode. cleared by N11 reading.
    B1     Ready:        Set after command decode competed.
    B2     Pulse Error:  Illegal parameters during pulse mode operation.
    B3     Error:        Illegel command or parameter have been received.
                         cleared by reading Error Status String N11.
    B6     Rqs           Set on Service Request mask enabled.
 */


// wait for serial poll ResByte to become 0x44, Data ready, rsv  
void StatusTest(char mask)
{
  char  ResByte = 0;	//one Byte only
  long  i_long;

  do {
   ibrsp(Dev, &ResByte);		// fetch device status
   if (*Argument == 'S') printf("0x%x ",ResByte);
   for (i_long = 0; i_long < 500000; ++i_long);		// delay
  }
  while ((ResByte & mask) != mask);		// Test for data ready 0x44
  if (*Argument == 'S') printf("\n");
}




/*
 Set waveform  and all other parameters
 ibcmd (Bd, "?&" , 2L);       //  UNL MLA6
 ibwrt (Dev, "U3\r", 3L);	// set 2040. U0 = disable, U1 = sinus, U3 = rectangular
 */
void SetWaveform(char* Waveform)
{
  int i;
/*
   STR[0]=0x3F;                      // UNL
   STR[1]=0x20+PRIMARY_ADDR_OF_DMM;   // MLAx
   STR[2]=0;                         // String termination
   ibcmd (Bd, STR, 2L);
*/
   strcpy(STR,Waveform);   // make a copy
   STR[strlen(Waveform)] = 0x0D;       // concatenate String terminator  \r
   STR[strlen(Waveform)+1] = 0;       // Terminate String
   ibwrt (Dev, STR, strlen(STR));

   StatusTest(0x2);
}




